//////////////////////////////////////////////////////////////////
//
//    Fries - Feature Retriever for Intensional Encoding of Sentences
//
//    Copyright (C) 2006   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This file is part of the Fries library
//
//    The Fries library is free software; you can redistribute it 
//    and/or modify it under the terms of the GNU General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    General Public License for more details.
//
//    You should have received a copy of the GNU General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110-1301 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx Omega.S112 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

//===========================================================
//=                                                         =
//=   This module is based on FEX project developed         =
//=   at U.Illinois, and distributed under LGPL             =
//=   with the permission of its authors                    =
//=                                                         =
//=   For info about original version, visit                =
//=      http://l2r.cs.uiuc.edu/~cogcomp/                   =
//=   or contact                                            =
//=      University of Illinois at Urbana/Champaign         =
//=      Department of Computer Science                     =
//=      Dr. Dan Roth - Cognitive Computation Group         =
//=                                                         =
//===========================================================

#include "fries/RGF.h"

#include <iostream>
#include <string>
#include <vector>
#include <algorithm>

using namespace std;

const char TARGET_STRING[] = "*";
const char TARGET_CHAR = '*';
const char FILLER_STRING[] = "_";
const char FILLER_CHAR = '_';
const char POS_STRING[] = "+";


/*****RGF Setup*****/

RGF::RGF() :
      subRGFs(SubRGF()),
      optParam(NULL),
      includeTarget(false),
      includeMark(false),
      targetIndex(TARGET_NULL),
      leftOffset(-1),
      rightOffset(-1),
      locationOffset(0),
      extractMode(EXTRACT_SENSOR),
      target(NULL),
      mask(NULL),
      genFeature(false),
      pSensor(NULL)
{};

/**********SENSOR REGISTRATION SECTION!!!!********/
/*THIS IS WHERE YOU NEED TO REGISTER YOUR SENSOR */
/*************************************************/
RGF::RGF(char* sensorName) :
      subRGFs(SubRGF()),
      optParam(NULL),
      includeTarget(false),
      includeMark(false),
      targetIndex(TARGET_NULL),
      leftOffset(-1),
      rightOffset(-1),
      locationOffset(0),
      extractMode(EXTRACT_SENSOR),
      target(NULL),
      mask(sensorName),
      genFeature(false)
{
  if (!strcmp (sensorName, "w")) pSensor = new SensorData("w");
  else if (!strcmp(sensorName, "t")) pSensor = new SensorData("t");
  else if (!strcmp(sensorName, "l")) pSensor = new SensorData("l");
  else if (!strcmp(sensorName, "pl")) pSensor = new SensorData("pl");
  else if (!strcmp(sensorName, "pt")) pSensor = new SensorData("pt");
  else if (!strcmp(sensorName, "na")) pSensor = new SensorData("na");
  else if (strlen(sensorName)>2 && sensorName[0]=='u' && sensorName[1]=='.') 
    pSensor = new SensorData(string(sensorName));
  else {
    cerr << "Invalid sensor identifier: " << sensorName << "!" << endl;
    exit(-1);
  }
}

RGF::RGF(char* sensorName, char* name, char* targ, char* parm, char* dataPath) :
      subRGFs(SubRGF()),
      optParam(NULL),
      includeTarget(false),
      includeMark(false),
      targetIndex(TARGET_NULL),
      leftOffset(-1),
      rightOffset(-1),
      locationOffset(0),
      extractMode(EXTRACT_SENSOR),
      target(NULL),
      mask(name),
      genFeature(false)
{
  if(!strcmp (sensorName, "map"))
    pSensor = new SensorMap(targ,parm,dataPath);
   else
     if(!strcmp (sensorName, "set"))
        pSensor = new SensorSet(targ,parm,dataPath);
   else
     if(!strcmp (sensorName, "setpart"))
        pSensor = new SensorSetPart(targ,parm,dataPath);
   else
     if(!strcmp (sensorName, "matchRE"))
        pSensor = new SensorMatchRE(targ,parm);
   else
     if(!strcmp (sensorName, "checkRE"))
        pSensor = new SensorCheckRE(targ,parm);
   else
     if(!strcmp (sensorName, "checkMwRE"))
        pSensor = new SensorCheckMwRE(targ,parm);
   else	
   {
      cerr << "Invalid sensor identifier: " << sensorName << "!" << endl;
      exit(-1);
   }
}


RGF::RGF( SubRGF& subR ) :
      subRGFs(subR),
      optParam(NULL),
      includeTarget(false),
      includeMark(false),
      targetIndex(TARGET_NULL),
      leftOffset(-1),
      rightOffset(-1),
      extractMode(EXTRACT_COLOC),
      target(NULL),
      mask(NULL),
      genFeature(false),
      pSensor(NULL)
{ }

RGF::~RGF()
{
//   if( optParam != NULL)
//      free(optParam);
}

void RGF::Mode( ExtractMode mode )
{
   extractMode = mode;

   if (mode == EXTRACT_LABEL)
      mask = "label";
}


void RGF::Target( char* targ )
{
     for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
          subRGFs.end(); pSub++)
        pSub->Target(targ);

	  if (isdigit(*targ) || (*targ == '-'))
     {
        targetIndex = atoi(targ);
        target = NULL;
     }
     else
     {
        if ((*targ == '\'') || (*targ == '"'))
        {
           target = targ++;
        } else
        {
           target = targ;
        }
	  }
}


void RGF::Param(char* val)
{
   if(val != NULL)
   {
      optParam = val;
   }
}


void RGF::IncludeTarget(bool val)
{
     for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
          subRGFs.end(); pSub++)
        pSub->IncludeTarget(val);

     includeTarget = val;
}

void RGF::IncludeMark(bool val)
{
     for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
          subRGFs.end(); pSub++)
        pSub->IncludeMark(val);

     includeMark = val;
}

void RGF::IncludeLocation(bool val)
{
     for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
          subRGFs.end(); pSub++)
        pSub->IncludeLocation(val);

     if(pSensor != NULL)
        pSensor->IncludeLocation(val);
}

void RGF::LocationOffset( int val )
{
     for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
          subRGFs.end(); pSub++)
        pSub->LocationOffset(val);

     locationOffset = val;
}


void RGF::RightOffset( int val )
{

  rightOffset = val;

  // Not check if ER:Relation
  //if (globalParams.erExtension && globalParams.labelType == RELATION)
  //  return;

  if(val < leftOffset) {
    cerr << "Invalid scope definition" << endl;
    exit(-1);
  }
}


void RGF::Show()
{
   cout << "               this: " << this << endl;
   if((long)optParam == -1)
      cout << "      optParam is -1" << endl;
   else
	   cout << "           optParam: " << optParam    << endl;
	cout << "        targetIndex: " << targetIndex << endl;
   if(pSensor != NULL)
      cout << "    includeLocation: " << pSensor->IncludeLocation() << endl;
	cout << "     locationOffset: " << locationOffset << endl;
	cout << "      includeTarget: " << includeTarget << endl;
	cout << "         leftOffset: " << leftOffset  << endl;
	cout << "        rightOffset: " << rightOffset << endl;
	cout << "        extractMode: " << extractMode << endl;
	cout << "             target: " << target      << endl;
   cout << "               mask: " << mask        << endl;
   cout << "    generic feature: " << genFeature  << endl;
   cout << "            pSensor: " << pSensor     << endl;
   for(SubRGF::iterator pRGF = subRGFs.begin(); pRGF != subRGFs.end(); pRGF++)
      cout << "             subRGF: " << &*pRGF << endl;
   cout << endl;

   for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
         subRGFs.end(); pSub++)
   {
      pSub->Show();
   }
}

/*
RGF* RGF::InMode(ExtractMode mode)
{
   RGF* val = 0;
   if(extractMode == mode)
      val = this;
   for(SubRGF::iterator pSub = subRGFs.begin(); pSub !=
         subRGFs.end(); pSub++)
      val = pSub->InMode(mode);
   return val;
}
*/

/*****Feature Extraction*****/

set<string> RGF::Extract( const vector<word>& sent, int targIndex )
{
   set<string> outputFeatures;

   // Set up Right scope index
   // realized i'm never using start and i can do the scoloc thing with it
   // keeping the variable just in case we end up needing it
   unsigned int start, end;
   if(rightOffset != RANGE_ALL)
   {
	   start = max(targIndex + leftOffset, 0);
	   end = min(targIndex + rightOffset, (int)sent.size());
      if(start > end)
      {
         return set<string>();
      }
   }
   else
   {
      start = 0;
      end = sent.size() - 1;
   }

   // Use Process method to extract features rooted at each record and
   // insert them into the output set
   for (unsigned int pos = start; pos <= end && pos < sent.size(); pos++)
   {
      set<string> Set = Process(sent, pos, targIndex, -1, end);
      for(set<string>::iterator pFeat = Set.begin();
            pFeat != Set.end(); pFeat++)
         if(*pFeat != TARGET_STRING)
            outputFeatures.insert(*pFeat);
   }
   return outputFeatures;
}


// A different version of Extract, used for phrase case only.
// Added by Scott Yih, 09/25/01
//  RawFeatureSet RGF::Extract( vector<word>& sent, int targIndex, int targLength )
//  {
//     RawFeatureSet outputFeatures;

//     if (extractMode == EXTRACT_CONJUNCT) {

//       outputFeatures = subRGFs[0].Extract(sent, targIndex, targLength);
//       if(!outputFeatures.empty())
//         for(int sub = 1; sub < subRGFs.size(); sub++)
//           {
//  	   RawFeatureSet inSet1 = outputFeatures;
//  	   RawFeatureSet inSet2;

//  	   inSet2 = subRGFs[sub].Extract(sent, targIndex, targLength);

//  	   outputFeatures.clear();
//  	   if(inSet2.empty())
//  	     break;

//  	   for(RawFeatureSet::iterator p1 = inSet1.begin();
//  	       p1 != inSet1.end(); p1++)
//  	     for(RawFeatureSet::iterator p2 = inSet2.begin();
//  		 p2 != inSet2.end(); p2++)
//                 {
//  		 string tempstr(*p1);
//  		 tempstr.append("--");
//  		 tempstr.append(*p2);
//  		 outputFeatures.insert(tempstr);
//                 }
//           }

//       return outputFeatures;
//     }

//     // Set up Right scope index
//     // realized i'm never using start and i can do the scoloc thing with it
//     // keeping the variable just in case we end up needing it
//     int start, end;

//     // Ignore RANGE_ALL
//     // preserve original leftOffset & rightOffset
//     int lOffset = leftOffset, rOffset = rightOffset;

//     if(rightOffset != RANGE_ALL) {
//       // This modification doesn't allow the window covers target phrase
//       if (lOffset > 0)   // right window
//         targIndex += targLength - 1;
//       else if (leftOffset == rightOffset && leftOffset == 0) { // target phrase
//         targIndex --;
//         lOffset ++;
//         rOffset += targLength;
//       }
//       // before - inside - after, only first word would be treated as target
//       else if (leftOffset < 0 && rightOffset > 0)
//         rOffset += targLength - 1;

//       start = max(targIndex + lOffset, 0);
//       end = min(targIndex + rOffset, (int)sent.size()-1);

//       // cerr << "start = " << start << " end = " << end << " targIndex = " << targIndex << endl;

//       if(start > end)
//         return RawFeatureSet();
//     }
//     else {
//       start = 0;
//       end = sent.size() - 1;
//     }

//     if (extractMode == EXTRACT_LABEL) // treated LABEL specially
//     {
//       // check the phrase label of words[targIndex..targIndex+targLength-1]
//       bool legitimate = false;
//       string label;

//       // check if the first word is the start of the phrase
//       if (sent[targIndex].phraseLabel[0] == 'B' &&
//  	 sent[targIndex].phraseLabel[1] == '-') {
//         legitimate = true;
//         label = sent[targIndex].phraseLabel.substr(2);
//       }

//       // check if other words are inside the phrase
//       for (int i = targIndex + 1; i < targIndex + targLength; i++) {
//         if (!legitimate)
//  	 break;

//         if (sent[i].phraseLabel[0] == 'I' &&
//  	   sent[i].phraseLabel[1] == '-') {
//  	 if (label != sent[i].phraseLabel.substr(2))
//  	   legitimate = false;
//         }
//         else
//  	 legitimate = false;
//       }

//       // check the word after is outside the phrase
//       if (legitimate && (targIndex + targLength <= end)) {
//         int next = targIndex + targLength;

//         if (sent[next].phraseLabel[0] == 'I' &&
//  	   sent[next].phraseLabel[1] == '-')
//  	 legitimate = false;
//       }

//       if (legitimate)
//         outputFeatures.insert(mask + string("[") + label + string("]"));
//       else
//         outputFeatures.insert(mask + string("[") + string("IRRELEVANT") + string("]"));
//     }
//     else if (extractMode == EXTRACT_SENSOR && pSensor->getSensorType() == ST_PHRASE) {
//       // Only process it when the range is [0,0]
//        if (leftOffset == rightOffset &&
//  	  leftOffset == 0 &&
//  	  pSensor != NULL) {

//  	// treat the range as the target indicator
//  	RawFeatureSet Set = Process(sent, start, targIndex, start, end);

//  	for(RawFeatureSet::iterator pFeat = Set.begin();
//  	    pFeat != Set.end(); pFeat++)
//  	  if(*pFeat != TARGET_STRING) {

//  	    // cerr << *pFeat << endl;

//  	    // if within target phrase and with location information, add '*' at last
//  	    if (pSensor->IncludeLocation())
//  	      outputFeatures.insert(*pFeat + TARGET_STRING);
//  	    else
//  	      outputFeatures.insert(*pFeat);
//  	  }
//        }
//     }
//     else {

//       // cerr << "start = " << start << " end = " << end << " targIndex = " << targIndex << endl;

//       // Use Process method to extract features rooted at each record and
//       // insert them into the output set
//       for (int pos = start; pos <= end; pos++) {
//         RawFeatureSet Set = Process(sent, pos, targIndex, -1, end);

//         for(RawFeatureSet::iterator pFeat = Set.begin();
//  	   pFeat != Set.end(); pFeat++)
//  	 if(*pFeat != TARGET_STRING) {

//  	   // cerr << *pFeat << endl;

//  	   // if within target phrase and with location information, add '*' at last
//  	   if ((leftOffset == rightOffset && leftOffset == 0) &&
//  	       (pSensor != NULL && pSensor->IncludeLocation()))
//  	       outputFeatures.insert(*pFeat + TARGET_STRING);
//  	   else
//  	     outputFeatures.insert(*pFeat);
//  	 }
//       }
//     }

//     return outputFeatures;
//  }


// Another different version of Extract, used for ER:Relation case only.
// Added by Scott Yih, 01/09/02
// RawFeatureSet RGF::ExtractRelation( vector<word>& sent, RelationInSentence& relSent, int posArg1, int posArg2 )
// {
//    RawFeatureSet outputFeatures;

//    if (extractMode == EXTRACT_CONJUNCT) {

//      outputFeatures = subRGFs[0].ExtractRelation(sent, relSent, posArg1, posArg2);
//      if(!outputFeatures.empty())
//        for(int sub = 1; sub < subRGFs.size(); sub++)
//          {
// 	   RawFeatureSet inSet1 = outputFeatures;
// 	   RawFeatureSet inSet2;

// 	   inSet2 = subRGFs[sub].ExtractRelation(sent, relSent, posArg1, posArg2);

// 	   outputFeatures.clear();
// 	   if(inSet2.empty())
// 	     break;

// 	   for(RawFeatureSet::iterator p1 = inSet1.begin();
// 	       p1 != inSet1.end(); p1++)
// 	     for(RawFeatureSet::iterator p2 = inSet2.begin();
// 		 p2 != inSet2.end(); p2++)
//                {
// 		 string tempstr(*p1);
// 		 tempstr.append("--");
// 		 tempstr.append(*p2);
// 		 outputFeatures.insert(tempstr);
//                }
//          }

//      return outputFeatures;
//    }

//    // Set up Right scope index
//    int start, end;

//    // preserve original leftOffset & rightOffset
//    int lOffset = leftOffset, rOffset = rightOffset;
//    // Reference arguments of leftOffset and rightOffset
//    int lRefArg, rRefArg;
//    // Reference origins of leftOffset and rightOffset
//    int lOrigin, rOrigin;

//    if (rightOffset != RANGE_ALL) {
//      lRefArg = abs(leftOffset) / 100;
//      rRefArg = abs(rightOffset) / 100;

//      //     if (globalParams.verbosity > VERBOSE_MIN) {
//      //       cerr << "leftOffset, rightOffset, lRefArg, rRefArg  = "
//      //	    << leftOffset << "," <<  rightOffset << "," << lRefArg << "," << rRefArg << endl;
//      //     }

//      lOrigin = (lRefArg == 1) ? posArg1 : posArg2;
//      rOrigin = (rRefArg == 1) ? posArg1 : posArg2;

//      lOffset = leftOffset % 100;
//      rOffset = rightOffset % 100;

//      start = max(lOrigin + lOffset, 0);
//      end = min(rOrigin + rOffset, (int)sent.size()-1);

//      //     if (globalParams.verbosity > VERBOSE_MIN) {
//      //       cerr << "leftOffset, rightOffset, lOrigin, rOrigin, lOffset, rOffset, start, end = "
//      //	    << leftOffset << "," <<  rightOffset << "," << lOrigin << "," << rOrigin << "," << lOffset << "," << rOffset << "," << start << "," << end << endl;
//      //     }

//      if(start > end)
//        return RawFeatureSet();
//    } else {
//      start = 0;
//      end = sent.size() - 1;
//    }

//    //    if (globalParams.verbosity > VERBOSE_MIN) {
//    //      cerr << "(extractMode == EXTRACT_LABEL) = " << (extractMode == EXTRACT_LABEL) << endl;
//    //    }

//    if (extractMode == EXTRACT_LABEL) // treated LABEL specially
//    {
// 	   bool hasRelation = false;
// 	   for (RelationInSentence::iterator pRelTag = relSent.begin();
// 	   		pRelTag != relSent.end(); pRelTag++) {

// 		  /*
// 		  cout << pRelTag->arg1 << ", " << pRelTag->arg2 << ", " << pRelTag->label << endl;
// 		  cout << posArg1 << ", " << posArg2 << endl;
// 		  */


// 		  if (pRelTag->arg1 == posArg1 && pRelTag->arg2 == posArg2) {

// 			  // cout << "Yes" << endl;

// 			  outputFeatures.insert(mask + string("[Rel-") + pRelTag->label + string("]"));
// 			  hasRelation = true;
// 		  }
// 	   }

// 	   if (!hasRelation)
// 	       outputFeatures.insert(mask + string("[Rel-Unknown]"));
//    } else {

//      // Use Process method to extract features rooted at each record and
//      // insert them into the output set
//      for (int pos = start; pos <= end; pos++) {
//        //RawFeatureSet Set = Process(sent, pos, lOrigin, -1, end);
//        RawFeatureSet Set = Process(sent, pos, lOrigin, start, end);

//        for(RawFeatureSet::iterator pFeat = Set.begin();
// 	   pFeat != Set.end(); pFeat++)
// 	 if(*pFeat != TARGET_STRING) {
// 	   if (includeMark) {
// 	       string lMark(1, char('0' + lRefArg)), rMark(1, char('0' + rRefArg));
// 	       outputFeatures.insert(*pFeat + lMark + rMark);
// 	   } else {
// 	       outputFeatures.insert(*pFeat);
// 	   }
// 	 }
//      }
//    }

//    return outputFeatures;
// }

set<string> RGF::Process( const vector<word>& sent,
                                 int rec,
                                 int targIndex,
                                 int start,
                                 int end)
{
   set<string> outSet;

   switch ( extractMode )
   {
   case EXTRACT_LABEL:
   {
      if(rec == targIndex)
      {
         set<string> tempSet =
            subRGFs[0].Process(sent, rec, targIndex, start, end);
         for(set<string>::iterator pFeat = tempSet.begin();
               pFeat != tempSet.end(); pFeat++)
         {
            outSet.insert(*pFeat);
         }
      }
      break;
   }
   case EXTRACT_CONJ:
   {
      //arg.. these first are going to be really confusing and stupid
      outSet = subRGFs[0].Process(sent, rec, targIndex, start, end);

      for(unsigned int sub = 1; sub < subRGFs.size(); sub++)
      {
         set<string> inSet1 = outSet;
         set<string> inSet2 = subRGFs[sub].Process(sent, rec,
               targIndex, start, end);

         outSet.clear();

         for(set<string>::iterator p1 = inSet1.begin();
               p1 != inSet1.end(); p1++)
            for(set<string>::iterator p2 = inSet2.begin();
                  p2 != inSet2.end(); p2++)
            {
               string tempstr(*p1);
               tempstr.append("&");
               tempstr.append(*p2);
               outSet.insert(tempstr);
            }
      }
      break;
   }
   case EXTRACT_DISJ:
   {
      for(SubRGF::iterator pRel = subRGFs.begin();
         pRel != subRGFs.end(); pRel++)
      {
         set<string> temp = pRel->Process(sent, rec, targIndex, start, end);
         set_union(outSet.begin(), outSet.end(),
                         temp.begin(),
                         temp.end(),
                         inserter(outSet,
                                  outSet.begin()));
      }
      break;
   }
   case EXTRACT_NOT:
   {
      set<string> tempoutSet;
      if(subRGFs.size() != 1 || 
          subRGFs.begin()->Mode() != EXTRACT_SENSOR ||
          subRGFs.begin()->Param() == NULL)
      {
         cerr << "Invalid argument to 'not' operator." << endl;
         exit(-1);             
      }
      if(subRGFs.begin() != subRGFs.end()){
         tempoutSet = subRGFs.begin()->Process(sent, rec, targIndex, start, end); 
         if(tempoutSet.empty())
         {
            string strparam = string(subRGFs.begin()->Param());
            string tmpstring = string(subRGFs.begin()->Mask()) + "[" +
                                 string(strparam) + "]";
            tempoutSet.insert(tmpstring);
         }
         else
            tempoutSet.clear();
      }
      else
         cerr << "something weird" << endl;

      outSet = tempoutSet;
      break;
   }
   case EXTRACT_COLOC:
   {
      outSet = subRGFs[0].Process(sent, rec, targIndex, start, end);
      if(!outSet.empty())
         for(int sub = 1; sub < (int)subRGFs.size(); sub++)
         {
            set<string> inSet1 = outSet;
            set<string> inSet2;
            if(rec+sub <= end && rec+sub < (int)sent.size())
               inSet2 = subRGFs[sub].Process(sent, rec+sub, targIndex,
                     start, end);
            outSet.clear();
            if(inSet2.empty())
               break;


            for(set<string>::iterator p1 = inSet1.begin();
                  p1 != inSet1.end(); p1++)
               for(set<string>::iterator p2 = inSet2.begin();
                     p2 != inSet2.end(); p2++)
               {
                  string tempstr(*p1);
                  tempstr.append("-");
                  tempstr.append(*p2);
                  outSet.insert(tempstr);
               }
         }
      break;
   }
   case EXTRACT_SCOLOC:
   {
      int left;
      if (start < 0)
         left = subRGFs.size()-1;
      else
         left = start;

      set<string> inSet1;
      inSet1 = subRGFs[subRGFs.size()-left-1].Process(sent, rec,
                                             targIndex, start, end);
      if(!inSet1.empty())
      {
         if(left)
         {
            // compute a safe end value
            int safeEnd = min((int)end, (int)sent.size() - 1);
            for (int j = rec+1; j <= safeEnd-left+1; j++)
            {
               set<string> inSet2 = this->Process(sent, j,
                     targIndex, left-1, end);
               for(set<string>::iterator p1 = inSet1.begin();
                     p1 != inSet1.end(); p1++)
                  for(set<string>::iterator p2 = inSet2.begin();
                        p2 != inSet2.end(); p2++)
                  {
                     string tempstr(*p1);
                     tempstr.append("-");
                     tempstr.append(*p2);
                     outSet.insert(tempstr);
                  }
            }
         }
         else
            outSet = inSet1;
      }
      break;
   }


   // I think this one is not a sensor, so I move this part of code here.
   // by Scott Wen-tau Yih
   case EXTRACT_LINK:
   {
      outSet = subRGFs[0].Process(sent, rec, targIndex, start, end);

      for(unsigned int sub = 1; sub < subRGFs.size(); sub++)
      {
         set<string> inSet1 = outSet;
         set<string> inSet2;

         int point;
	 //         if(sent[rec].pointer.begin() != sent[rec].pointer.end())
	 //            point = *(sent[rec].pointer.begin());
	 //         else
         {
            outSet.clear();
            return outSet;
         }

         inSet2 = subRGFs[sub].Process(sent, point, targIndex,
                  start, end);

         outSet.clear();

         for(set<string>::iterator p1 = inSet1.begin();
               p1 != inSet1.end(); p1++)
            for(set<string>::iterator p2 = inSet2.begin();
                  p2 != inSet2.end(); p2++)
            {
               string tempstr(*p1);
               tempstr.append("->");
               tempstr.append(*p2);
               outSet.insert(tempstr);
            }

         rec = point;
      }
      break;
   }

   // these are much easier/better
   // Below are sensors (Scott Wen-tau Yih)
   // slightly modified for generality -cmc11/14
   case EXTRACT_SENSOR:
	{
	  // Added by Scott Yih, 09/27/01
	  if (pSensor->getSensorType() == ST_PHRASE)
	    pSensor->Extract(sent, outSet, start, end);
	  else
	    pSensor->Extract( sent,
			      outSet,
			      rec,
			      rec - targIndex - locationOffset);

	  for(set<string>::iterator pFeat = outSet.begin(); pFeat != outSet.end();
	      pFeat++)
	    if(optParam != NULL)
	      {
		if(*pFeat != string(optParam))
		  outSet.erase(pFeat);
	      }
	  if(rec == targIndex && !includeTarget)
	    {
	      outSet.clear();
	      outSet.insert(TARGET_STRING);
	    }
	  break;
	}

   case EXTRACT_CONJUNCT:
       {
	 // missing
        break;
       }
   }

   set<string> tempSet = outSet;
   outSet.clear();
   for(set<string>::iterator pFeat = tempSet.begin();
      pFeat != tempSet.end(); pFeat++)
   {
      string feat;
      if(mask != NULL && *pFeat != TARGET_STRING)
      {
         feat += mask;
         feat += "[";
      }
      if(!genFeature)
         feat += *pFeat;
      if(mask != NULL && *pFeat != TARGET_STRING)
         feat += "]";
      outSet.insert(feat);
   }
   return outSet;
}

