/*
//////////////////////////////////////////////////////////////////
//
//    Fries - Feature Retriever for Intensional Encoding of Sentences
//
//    Copyright (C) 2006   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This file is part of the Fries library
//
//    The Fries library is free software; you can redistribute it 
//    and/or modify it under the terms of the GNU General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    General Public License for more details.
//
//    You should have received a copy of the GNU General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110-1301 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx Omega.S112 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


//===========================================================
//=                                                         =
//=   This module is based on FEX project developed         =
//=   at U.Illinois, and distributed under LGPL             =
//=   with the permission of its authors                    =
//=                                                         =
//=   For info about original version, visit                =
//=      http://l2r.cs.uiuc.edu/~cogcomp/                   =
//=   or contact                                            =
//=      University of Illinois at Urbana/Champaign         =
//=      Department of Computer Science                     =
//=      Dr. Dan Roth - Cognitive Computation Group         =
//=                                                         =
//===========================================================
*/

/*********************************************************************
*   This file is used to generate features_l.c with the 'flex' scanner
*  generator. The FreeLing building procedure will use the  features_l.c
*  file, not this one, in order not to force users to install flex.
* 
*   This file is included only in case you want to modify the program,
*  you know flex, and you know what you're doing. 
*  Take into account that this file depends on "readFeatures.h" 
*  and "tok_features.h", both in src/include
*
*   If you don't intend to modify the feature definition language, you need
*  neither to look at this file, nor to have flex installed
*
*   If you do, the features_l.cc file was generated with the 
*  following command:
*
*     flex -t features.l >features_l.cc
***/

%{
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "fries/readFeatures.h"

int lineno = 1;
%}

delim       [ \t]
ws          {delim}+
letter      [A-Za-z_\-]
nat         [0-9]+
int         -?{nat}
word        {letter}+
alphanum    {nat}|{letter}
targ        ({alphanum}|[\$\-\+\#\@\!])+
filenam     ([^"]+)

%s std flag arg quot

%%
<INITIAL>[ \t\n]+                             {} /* eat up whitespace */
<INITIAL,flag,std,arg>{ws}                    {}
<INITIAL,flag,std,arg>"/*"([^*]|\*[^/])*"*/"  {}
<INITIAL,flag,std,arg>"\n"                    {lineno++; BEGIN(INITIAL); return(NEWLINE);}
<std>coloc                           {yylval.i = T_COLOC; return(COMPRGF);}
<std>not                             {yylval.i = T_NOT; return(COMPRGF);}
<std>scoloc                          {yylval.i = T_SCOLOC; return(COMPRGF);}
<std>lab                             {yylval.i = T_LABEL; return(COMPRGF);}
<std>link                            {yylval.i = T_LINK; return(COMPRGF);}
<INITIAL>conjunct                    {return(CONJUNCT);}
<std>{word}("."{nat})?               {BEGIN(arg); 
                                      yylval.c = (char *) malloc(yyleng+1);
                                      strcpy(yylval.c, yytext);
                                      return(SENSOR);}
<flag>loc                            {return(FLOC);}
<flag>inc                            {return(FINC);}
<flag>mark                           {return(FMARK);}
<INITIAL>{targ}                      {BEGIN(flag);
                                      yylval.c = (char *) malloc(yyleng+1); 
                                      strcpy(yylval.c, yytext); 
                                      return(TARG); }
<flag>":"                            {BEGIN(std); return(COLON);}
<flag>"="                            {BEGIN(std); return(EQUALS);}
<std,arg>","                         {BEGIN(std); return(COMMA);}
<std,arg>";"                         {BEGIN(INITIAL); return(SEMICOLON);}
<std,arg>"["                         {BEGIN(std); return(LBRACK);}
<std>"]"                             {return(RBRACK);}
<INITIAL,std,arg>"("                 {return(LPAREN);}
<std,arg>"="                         {return(EQUALS);}
<std,arg>")"                         {BEGIN(std); return(RPAREN);}
<std,arg>"|"                         {BEGIN(std); return(BAR);}
<std,arg>"&"                         {BEGIN(std); return(AMP);}
<std>\"                              {BEGIN(quot); return(QUOTE);}
<quot>\"                             {BEGIN(std); return(QUOTE);}
<quot>{filenam}                      {yylval.c = (char *) malloc(yyleng+1);
                                      strcpy(yylval.c, yytext);
                                      return(TARG); }
<arg>x                               {return(SMALLX);}
<arg>X                               {return(BIGX);}
<arg>{targ}                          {yylval.c = (char *) malloc(yyleng+1);
                                      strcpy(yylval.c, yytext);
                                      return(TARG); }
<std,flag>{int}                      {yylval.i = atoi(yytext); return(INT);}
.                                    {return(ERROR);}

%%

int yywrap() { return 1; };
