//////////////////////////////////////////////////////////////////
//
//    Fries - Feature Retriever for Intensional Encoding of Sentences
//
//    Copyright (C) 2006   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This file is part of the Fries library
//
//    The Fries library is free software; you can redistribute it 
//    and/or modify it under the terms of the GNU General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    General Public License for more details.
//
//    You should have received a copy of the GNU General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110-1301 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx Omega.S112 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

//===========================================================
//=                                                         =
//=   This module is based on FEX project developed         =
//=   at U.Illinois, and distributed under LGPL             =
//=   with the permission of its authors                    =
//=                                                         =
//=   For info about original version, visit                =
//=      http://l2r.cs.uiuc.edu/~cogcomp/                   =
//=   or contact                                            =
//=      University of Illinois at Urbana/Champaign         =
//=      Department of Computer Science                     =
//=      Dr. Dan Roth - Cognitive Computation Group         =
//=                                                         =
//===========================================================

/********************************************************************
*   This file is used to generate features_y.c with the 'bison' parser
*  generator. The FreeLing building procedure will use the  features_y.c
*  file, not this one, in order not to force users to install bison.
* 
*   This file is included only in case you want to modify the program,
*  you know bison, and you know what you're doing. 
*  Take into account that this file depends on "readFeatures.h" 
*  and "tok_features.h", both in src/include
*
*   If you don't intend to modify the feature definition language, you need
*  neither to look at this file, nor to have flex installed
*
*   If you do, the features_y.c file was generated with the 
*  following commands:
*
*     bison -o features_y.cc --defines=tok_features.h features.y
*     mv tok_features.h ../include/fries/

***/


%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <vector>
#include <map>

#include "fries/readFeatures.h"

using namespace std;

static SubRGF *script;
static map<string, RGF> Mnemonics;

extern int lineno;
extern FILE *yyin;
int yylex();
int yyerror(char const *msg) {fprintf(stderr, "\nerror: on line %d: %s\n", lineno, msg); return 0;}

char* dataPath;
%}

%union {
  int i;      /* For returning numbers  */
  SubRGF *s;  /* For returning SubRGF*  */
  RGF *r;     /* For returning RGF*     */
  char *c;    /* For returning char*    */
}

%token <c> SENSOR 
%token <c> TARG
%token <i> COMPRGF 
%token <i> INT
%token CONJUNCT WORD FLAG COLON LBRACK RBRACK LPAREN RPAREN NEWLINE 
%token COMMA AMP BAR SMALLX BIGX EQUALS SEMICOLON
%token FLOC FINC FMARK ERROR QUOTE

%type <c> OptArg
%type <s> LineList
%type <s> LineItemList
%type <s> Stmt
%type <r> Line
%type <r> LineItem
%type <r> Disj
%type <r> Conj
%type <r> Expr
%type <i> FlagList
%type <i> FlagList2
%type <i> LocPart
%type <i> LeftOff
%type <i> RightOff

%start File

%%

File    : LineList
          {
             script = (SubRGF*)$1; 
          };

LineList: Line NEWLINE LineList        
         {
	   if ($1) {
               if ($1->Mode() == EXTRACT_LABEL)
                  $3->insert($3->begin(),*$1);
               else {
                  $3->push_back(*$1);
//                  delete (RGF*)$1;
               }   
	   }
            $$ = $3;
         } 
        | { 
            $$ = new SubRGF(); 
          };

Line: CONJUNCT LPAREN LineItemList RPAREN
      {
	 RGF *compExpr = new RGF(*$3);
	 compExpr->Mode(EXTRACT_CONJUNCT);
	 $$ = compExpr;
      }      
    | LineItem
      {
	 $$ = $1;
      };

LineItemList: LineItemList SEMICOLON LineItem
              {
                 $1->push_back(*$3);
                 $$ = $1;
              }
            | LineItem
              {
		 SubRGF* sub = new SubRGF();
		 sub->push_back(*$1); 
		 $$ = sub; 
	      };

LineItem: TARG FlagList COLON Disj LeftOff RightOff    
         {
            $4->Target($1); 
            $4->LeftOffset($5); 
            $4->RightOffset($6); 

	    if ($2 >= 8000) {
	      $2 = $2 - 8000;
	      $4->IncludeMark(true);
	    }

            if($2 != 3000) 
            {
               if($2 == 2000)
                  $4->IncludeTarget(true); 
               else
               {
                  if($2 >= 500)
                     { 
                        $4->IncludeLocation(true);
                        $4->LocationOffset($2-1000);
                        $4->IncludeTarget(true);
                     }
                  else
                  {
                     $4->IncludeLocation(true);
                     $4->LocationOffset($2);
                  }
               }
            }
            $$ = $4; 
         }

        | TARG EQUALS Disj
         {
            $3->Mask($1);
            Mnemonics.insert(pair<string, RGF>($1, *$3));
            $$ = 0; 
         };
                
Stmt    : Stmt COMMA Disj 
          {
             $1->push_back(*$3);
//             delete (RGF*)$3;
             $$ = $1;
          }
        | Disj              
          {
             SubRGF* sub = new SubRGF();
             sub->push_back(*$1); 
//             delete (RGF*)$1;
             $$ = sub; 
          };
          
Disj    : Disj BAR Conj
          {  
             if($1->Mode() == EXTRACT_DISJ)
             {
               $1->Insert(*$3);
               $$ = $1;
             }
             else
             { 
               RGF* disj = new RGF();
               disj->Mode(EXTRACT_DISJ);
               disj->Insert(*$1);
               disj->Insert(*$3);
               $$ = disj;
             }
          }
        | Conj 
          {
             $$ = $1;
          };

Conj    : LPAREN Conj RPAREN
          { 
             $$ = $2; 
          }
        | Conj AMP Expr
          { 
             if ($1->Mode() == EXTRACT_CONJ)
             {
               $1->Insert(*$3);
               $$ = $1;
             }
             else
             {
               RGF* conj = new RGF();
               conj->Mode(EXTRACT_CONJ); 
               conj->Insert(*$1);
               conj->Insert(*$3);
               $$ = conj;
             }
          }
        | Expr
          {    
             $$ = $1; 
          };

Expr    : SENSOR OptArg 
          { 
            RGF *primExpr;
            // try to find it in the mnemonics map first
            if(Mnemonics.find($1) != Mnemonics.end())
            {
               primExpr = new RGF(Mnemonics[$1]);
            }
            else
            {
               //create Sensor RGF obj for this expression
               primExpr = new RGF($1);
            }

            //set the generic/specific feature mode
            if ($2 < 0)
               primExpr->GenFeature(true);
            else
               if ($2)
                  primExpr->Param($2);

             $$ = primExpr;
          }
        | SENSOR LPAREN TARG COMMA SENSOR COMMA QUOTE TARG QUOTE RPAREN
          { 
            RGF *primExpr;
	    //create Sensor RGF obj for this expression
	    primExpr = new RGF($1,$3,$5,$8,dataPath);
	    $$ = primExpr;
          }
        | COMPRGF LPAREN Stmt RPAREN 
          { 
             RGF *compExpr = new RGF(*$3);
             switch ($1)
             {
             case T_COLOC:
                compExpr->Mode(EXTRACT_COLOC);
                break;
             case T_SCOLOC:
                compExpr->Mode(EXTRACT_SCOLOC);
                break;
             case T_LINK:
                compExpr->Mode(EXTRACT_LINK);
                break;
             case T_LABEL:
                compExpr->Mode(EXTRACT_LABEL);
                compExpr->IncludeTarget(true);
                break;
             case T_NOT:
                compExpr->Mode(EXTRACT_NOT);
                compExpr->Mask("not");
                break;
             }
             $$ = compExpr;
          }
         | COMPRGF 
           {
	     // no parenthese and other sensors after "lab"
	     // used only for phrase case
	     // Added by Scott Yih, 09/27/01
             if ($1 == T_LABEL) {
	       RGF *compExpr = new RGF();
	       compExpr->Mode(EXTRACT_LABEL);
	       compExpr->IncludeTarget(true);
	       $$ = compExpr;
	     }
	   };

FlagList : FMARK FlagList2
           { $$ = $2 + 8000; }
        | FlagList2
           { $$ = $1; };

FlagList2 : FINC LocPart 
            { $$ = $2 + 1000; }
         | LocPart
            { $$ = $1; }
         | FINC 
            { $$ = 2000; }
         | 
            { $$ = 3000; };

LocPart : FLOC INT
          { $$ = $2; }
        | FLOC
          { $$ = 0; };

OptArg : LPAREN BIGX RPAREN        { $$ = (char *) -1; } 
         | LPAREN SMALLX EQUALS TARG RPAREN
                                   { $$ = $4; }
         |                         { $$ = 0; }     ;             

LeftOff : LBRACK INT  { $$ = $2; }
         |             { $$ = RANGE_ALL; };
        
RightOff : COMMA INT RBRACK  { $$ = $2; }
         |                   { $$ = RANGE_ALL;};

%%

SubRGF* DoParse(const char *inFile) {
    int result; 
    char *p;

    // open file
    if ((yyin = fopen(inFile, "r")) == NULL) {
        fprintf(stderr, "could not open %s for input.\n", inFile);
        exit(1);
    }

    // store file path, to use it as prefix for data files
    dataPath = new char[strlen(inFile)+1];
    strcpy(dataPath,inFile);
    p=strrchr(dataPath,'/'); 
    if (p) (*(p+1))=0; // remove file name, keep only path
    else dataPath[0]=0;

    result = yyparse(); // parse file

    fclose(yyin);       // close file

    delete dataPath;  // free space

    // return appropriate result
    if(!result) return script;
    else return 0;
}


