//////////////////////////////////////////////////////////////////
//
//    Fries - Feature Retriever for Intensional Encoding of Sentences
//
//    Copyright (C) 2006   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This file is part of the Fries library
//
//    The Fries library is free software; you can redistribute it 
//    and/or modify it under the terms of the GNU General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    General Public License for more details.
//
//    You should have received a copy of the GNU General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110-1301 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx Omega.S112 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

using namespace std;

//===========================================================
//=                                                         =
//=   This module is based on FEX project developed         =
//=   at U.Illinois, and distributed under LGPL             =
//=   with the permission of its authors                    =
//=                                                         =
//=   For info about original version, visit                =
//=      http://l2r.cs.uiuc.edu/~cogcomp/                   =
//=   or contact                                            =
//=      University of Illinois at Urbana/Champaign         =
//=      Department of Computer Science                     =
//=      Dr. Dan Roth - Cognitive Computation Group         =
//=                                                         =
//===========================================================

#include "fries/sensor.h"

#include <string>
#include <vector>
#include <set>
#include "fries/util.h"
#include <list>

using namespace std;

//--------------------------------------------------------------

const char TARGET_STRING[] = "*";
const char TARGET_CHAR = '*';
const char FILLER_STRING[] = "_";
const char FILLER_CHAR = '_';
const char POS_STRING[] = "+";

//--------------------------------------------------------------


// feat is the string for lexicon; checkFeat is the string for internal checking
void Sensor::Output( std::set<std::string> &outSet, std::string feat, int loc) {

  if (includeLocation) {
    static const char filler = FILLER_CHAR;
    
    // Make a local copy so that primitive and lex may
    // reference the same string.
    string temp(feat);
    
    // clear out the existing lexical string
    feat.resize(0);
    
    if (loc > 0) {
      feat.append(1, TARGET_CHAR);
      feat.append(loc - 1, filler);
    }
    feat.append(temp);
    if (loc < 0) {
	feat.append(-loc - 1, filler);
	feat.append(1, TARGET_CHAR);
    }
  }
  
  if (feat != "") outSet.insert(feat);
}


//--------------------------------------------------------------

void SensorData::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {
  list<string> feat=solve_target(sent,rec);
  for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++)
    Output(outSet, *f, targLoc);
}


//--------------------------------------------------------------

void SensorMap::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {
   map<string,string>::iterator it;
   list<string> feat=solve_target(sent,rec);
   for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++) {
     it = content.find(*f);
     if (it != content.end()) Output(outSet, it->second, targLoc);   
   }
}

//--------------------------------------------------------------

void SensorSet::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {
   set<string>::iterator it;
   list<string> feat=solve_target(sent,rec);
   for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++) {
     it = content.find(*f);       
     if (it != content.end()) Output(outSet, "x", targLoc);   
   }
}

//--------------------------------------------------------------

void SensorSetPart::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {
  list<string> feat=solve_target(sent,rec);
  for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++){
    string s = *f;
    list<string> l = util::string2list(s, "_");
    bool trobat = l.size() <= 1;
    list<string>::iterator lit = l.begin();
    while (lit != l.end() and not trobat) {
      set<string>::iterator sit = content.find(*lit);
      if (sit != content.end()) {
	Output(outSet, "x", targLoc);
	trobat =  true;
      }
      lit++;
    }
  }
}


//--------------------------------------------------------------

void SensorMatchRE::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {
  list<string> feat=solve_target(sent,rec);
  for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++)
   if (expression.Search(f->c_str())) Output(outSet, expression.Match(0), targLoc);   
}


//--------------------------------------------------------------

void SensorCheckMwRE::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {

  list<string> feat=solve_target(sent,rec);
  for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++) {

    string ins = *f; //recupera la paraula, el lema o el tag
   
   // a list<string> mw_patrons tenim tots els patrons que volem reconixer en forma "N,RegExpr1"..."N,RegExpr2"
   // la costrucci ser,  x="Pi", el primer patro que la paraula compleixi (es recomana posar-los de mes 
   // a menys especific a la regla...)
   // y = x1_x2_x3..xN per totes les paraules
   // A output hi anir targLoc[x]
   
   string name, expr, y="";
   list<string> patro;
   list<string> l_words;
 
   l_words = util::string2list (ins, "_");
   list<string>::iterator it_lw= l_words.begin();

   while ( it_lw != l_words.end() )
     {
       string c_word = (*it_lw);

       //busca patrons de la paraula
       list<string>::iterator patronsIt = mw_patrons.begin();
       string x = "";

       bool trobat=false;
       while ( patronsIt != mw_patrons.end() && !trobat ) 
	 {
	   patro = util::string2list(*patronsIt, "=");
	   // patro hauria de tenir exactament 2 elements: N, RegExpr
	   if ( patro.size() == 2 )
	     {
	       list<string>::iterator patroIt = patro.begin();
	       name = string( *patroIt );
	       patroIt++;
	       expr = string( *patroIt );
	       
	       // si compleix la expr regular afegeix la marca de patro
	       RegEx expression(expr.c_str());
	       if (expression.Search(c_word.c_str())) {
                 x = (name);
		 trobat = true;
	       }
	     }
	   else {
	     cerr<<"FEX: wrong pattern in MwRE rule "<<(*patronsIt)<<endl;
	   }

	   patronsIt++;
	   patro.clear();
	 } //fi de patrons de la paraula

       // no complia cap patro
       if ( !trobat ) x="*";
       // acumular patro
       y += x + "_";
       // seguent paraula
       it_lw++;
     }
   
   Output(outSet, y.c_str(), targLoc);    
  }
}


//--------------------------------------------------------------
void SensorCheckRE::Extract(const vector<word> &sent, std::set<std::string> &outSet, int rec, int targLoc) {
  list<string> feat=solve_target(sent,rec);
  for (list<string>::iterator f=feat.begin(); f!=feat.end(); f++)
   if (expression.Search(f->c_str())) Output(outSet, "x", targLoc);
}

