//////////////////////////////////////////////////////////////////
//
//    Fries - Feature Retriever for Intensional Encoding of Sentences
//
//    Copyright (C) 2006   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This file is part of the Fries library
//
//    The Fries library is free software; you can redistribute it 
//    and/or modify it under the terms of the GNU General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    General Public License for more details.
//
//    You should have received a copy of the GNU General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110-1301 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx Omega.S112 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////

#include <iostream>
#include "fries/util.h"


#include <sstream>
#include <iomanip>  

using namespace std;

/////////////////////////////////////////////////////////////////////////////
/// Lowercase an string, possibly with accents.
/////////////////////////////////////////////////////////////////////////////

string util::lowercase(const string &p) {
  string::iterator pos;

  string s=p;
  for (pos=s.begin(); pos!=s.end(); pos++) {
    if (*pos=='') *pos='';
    else if (*pos=='') *pos='';         
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';  
    else if (*pos=='') *pos=''; 
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';   
    else if (*pos=='') *pos='';  
    else if (*pos=='') *pos='';
    else if (isalpha(*pos)) *pos=tolower(*pos);
  }           
  return(s);
}

/////////////////////////////////////////////////////////////////////////////
/// Find out whether a string contains lowercawse chars
/////////////////////////////////////////////////////////////////////////////

bool util::has_lowercase(const string &p) {
  string::const_iterator pos;

  bool lc=false;
  for (pos=p.begin(); pos!=p.end() && !lc; pos++) {
    lc =  (*pos>='a' && *pos<='z') || *pos=='' || *pos=='' || *pos=='' || *pos=='' 
        || *pos=='' || *pos=='' || *pos=='' || *pos=='' || *pos=='' || *pos=='' 
        || *pos=='' || *pos=='' || *pos=='' || *pos=='';
  }           
  return(lc);
}

/////////////////////////////////////////////////////////////////////////////
/// Find out whether a char is alphanum, considering latin characters
/////////////////////////////////////////////////////////////////////////////

bool util::isalphanum(const char c) {
  return (isalnum(c) 
         || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' 
         || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' || c==''
         || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' || c==''
         || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' || c=='');
}

/////////////////////////////////////////////////////////////////////////////
/// Find out whether a char is uppercase, considering latin characters
/////////////////////////////////////////////////////////////////////////////

bool util::isuppercase(const char c) {
  return (isupper(c) 
	  || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' || c==''
	  || c=='' || c=='' || c=='' || c=='' || c=='' || c=='' || c=='');
}



/////////////////////////////////////////////////////////////////////////////
/// Find out whether a char is alphanum, considering latin characters
/////////////////////////////////////////////////////////////////////////////

bool util::isanumber(const string &p) {
  string::const_iterator pos;

  bool in=true;
  for (pos=p.begin(); pos!=p.end() && in; pos++) {
    in = (*pos>='0' && *pos<='9');
  }           
  return(in && p.size()>0);
}


/// Type conversion

int util::string2int(const string &s) {
istringstream sst_in;	
int x;
string r;

  sst_in.str(s);
  sst_in>>x;
  if (sst_in>>r) x= -99999;  // if original string hasn't been fully emptied
  return x;
}

/// Type conversion

string util::int2string(const int x) {
ostringstream sst_out;	
string s;
  sst_out<<x;
  s=sst_out.str();
  return s;
}

/// Type conversion

double util::string2double(const string &s) {
istringstream sst_in;	
double x;
  sst_in.str(s);
  sst_in>>x;
  return x;
}

/// Type conversion

string util::double2string(const double x) {
ostringstream sst_out;	
string s;
  sst_out<<fixed<<x;
  s=sst_out.str();
  return s;
}

/// Type conversion

long double util::string2longdouble(const string &s) {
istringstream sst_in;	
double x;
  sst_in.str(s);
  sst_in>>x;
  return x;
}

/// Type conversion

string util::longdouble2string(const long double x) {
ostringstream sst_out;	
string s;
string::iterator i;

  sst_out<<fixed<<x;
  s=sst_out.str();
  // ignore decimal digits if all zeros.
  for (i=s.end(), i--; (*i)=='0'; i--) s.erase(i);
  if ((*i)=='.') s.erase(i);

  return s;
}

/////////////////////////////////////////////////////////////////////////////
/// Create a single string concatenatig all strings in given 
/// list with given separator 
/////////////////////////////////////////////////////////////////////////////

string util::list2string(const list<string> &ls, const string &sep) {
  string sn;

  // concatenate strings in list
  list<string>::const_iterator i;
  for (i=ls.begin(); i!=ls.end(); i++) {
    sn=sn+sep+(*i);
  }

  // erase separator before first element
  if (sn.length()>0) sn.erase(0,sep.size());

  return(sn);
}


/////////////////////////////////////////////////////////////////////////////
/// Create a single string concatenatig all strings in given 
/// vector with given separator 
/////////////////////////////////////////////////////////////////////////////

string util::vector2string(const vector<string> &ls, const string &sep) {
  string sn;

  // concatenate strings in list
  vector<string>::const_iterator i;
  for (i=ls.begin(); i!=ls.end(); i++) {
    sn=sn+sep+(*i);
  }

  // erase separator before first element
  if (sn.length()>0) sn.erase(0,sep.size());

  return(sn);
}


/////////////////////////////////////////////////////////////////////////////
/// Create a single string concatenatig all elements in given 
/// list with given separators (one for list elements, one for pair elements)
/////////////////////////////////////////////////////////////////////////////

string util::pairlist2string(const list<pair<string, double> > &ls, const string &sep_pair, const string &sep_list) {
  string sn;

  // concatenate strings in list
  list<pair<string, double> >::const_iterator i;
  for (i=ls.begin(); i!=ls.end(); i++) {
    sn= sn+sep_list+(i->first)+sep_pair+util::double2string(i->second);
  }

  // erase separator before first element
  if (sn.length()>0) sn.erase(0,sep_list.size());

  return(sn);
}


/////////////////////////////////////////////////////////////////////////////
/// Split a string into a list of strings given a separator substring
/////////////////////////////////////////////////////////////////////////////

list<string> util::string2list(const string &s, const string &sep){

  list<string> ls;
  string::size_type p,q;

  p=0; q=s.find(sep);
  while(q!=string::npos){
    ls.push_back(s.substr(p,q-p));
    p=q+sep.size();
    q=s.find(sep,p);
  }

  if (s!="") ls.push_back(s.substr(p,s.size()-p));

  return(ls);
}

/////////////////////////////////////////////////////////////////////////////
/// Slpit a string into a vector of strings given a separator substring
/////////////////////////////////////////////////////////////////////////////

vector<string> util::string2vector(const string &s, const string &sep){

  vector<string> vs;
  string::size_type p,q;

  p=0; q=s.find(sep);
  while(q!=string::npos){
    vs.push_back(s.substr(p,q-p));
    p=q+sep.size();
    q=s.find(sep,p);
  }

  if (s!="") vs.push_back(s.substr(p,s.size()-p));

  return(vs);
}


/////////////////////////////////////////////////////////////////////////////
/// Split a string into a set of strings given a separator substring
/////////////////////////////////////////////////////////////////////////////

set<string> util::string2set (const string &s, const string &sep){

  set<string> ls;
  string::size_type p,q;

  p=0; q=s.find(sep);
  while(q!=string::npos){
    ls.insert(s.substr(p,q-p));
    p=q+sep.size();
    q=s.find(sep,p);
  }

  if (s!="") ls.insert(s.substr(p,s.size()-p));

  return(ls);
}

/////////////////////////////////////////////////////////////////////////////
/// Create a single string concatenatig all strings in given 
/// set with given separator.
/////////////////////////////////////////////////////////////////////////////

string util::set2string(const set<string> &ls, const string &sep) {
  string sn;

  // concatenate strings in list
  set<string>::const_iterator i;
  for (i=ls.begin(); i!=ls.end(); i++) {
    sn=sn+sep+(*i);
  }

  // erase separator before first element
  if (sn.length()>0) sn.erase(0,sep.size());

  return(sn);
}

/////////////////////////////////////////////////////////////////////////////
/// convert a relative path to absolute.
/////////////////////////////////////////////////////////////////////////////

string util::absolute(const string &fname, const string &path)  {
  string fn=fname;

  // remove " quotes around filename, if any
  if (fn[0]=='"' && fn[fn.size()-1]=='"') fn=fn.substr(1,fn.size()-2);   
  // prepend given path if filename is relative
  if (fn.size()>0 && fn[0]!='/' && fn[0]!='\\') fn=path+fn;

  return fn;
}

/////////////////////////////////////////////////////////////////////////////
/// Auxliliar function: delete all the chars in text presents in clist.
/////////////////////////////////////////////////////////////////////////////

string util::eliminateChars(const string &text, const string &clist){ 

  string s;

  string::size_type i=0;
  for (string::size_type j=0; j<text.size(); j++) {
    // if text[j] is to be erased
    if (clist.find(text[j])!=string::npos) {
      // add the previous substring to the result
      s = s+ text.substr(i,j-i);
      // update start, skipping separator
      i=j+1; 
    }
  }
  s = s+ text.substr(i,text.size()-i);
  return (s);
}

/////////////////////////////////////////////////////////////////////////////
/// Split a string into a vector of strings given a list of separator 
/// chars presents in sep1.
/////////////////////////////////////////////////////////////////////////////

vector<string> util::split(const string &text, const string &sep){
  
  vector<string> vs;

  string::size_type i=0;
  for (string::size_type j=0; j<text.size(); j++) {
    // if text[j] is a separator
    if (sep.find(text[j])!=string::npos) {
      // add the previous substring to the vector
      vs.push_back(text.substr(i,j-i));
      // update start, skipping separator
      i=j+1;      
    }  
  }

  vs.push_back(text.substr(i,text.size()-i));
  return(vs);
}


/////////////////////////////////////////////////////////////////////////////
/// Replace all occurrences of s in t by r
/////////////////////////////////////////////////////////////////////////////

void util::find_and_replace(string &t, const string &s, const string &r) {
   size_t p=t.find(s); 
   while (p!=string::npos) {
     t.replace(p,s.size(),r);
     p=t.find(s,p+r.size());
   }
}

