//////////////////////////////////////////////////////////////////
//
//    Fries - Feature Retriever for Intensional Encoding of Sentences
//
//    Copyright (C) 2006   TALP Research Center
//                         Universitat Politecnica de Catalunya
//
//    This file is part of the Fries library
//
//    The Fries library is free software; you can redistribute it 
//    and/or modify it under the terms of the GNU General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    General Public License for more details.
//
//    You should have received a copy of the GNU General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110-1301 USA
//
//    contact: Lluis Padro (padro@lsi.upc.es)
//             TALP Research Center
//             despatx Omega.S112 - Campus Nord UPC
//             08034 Barcelona.  SPAIN
//
////////////////////////////////////////////////////////////////


//------------------------------------------------------------------//
//
//                    IMPORTANT NOTICE
//
//  This file contains a sample main program to illustrate 
//  usage of Fries feature-extraction library.
//
//  Note that if you want embed Fries libraries inside a larger
//  application, or you want to deal with other input/output formats
//  (e.g. XML,HTML,...), the efficient and elegant way to do so is
//  consider this file as a mere example, and call the library from
//  your your own main code.
//
//------------------------------------------------------------------//


// ####  Convert a corpus into feature vectors (to stdout)
// ####  and produce a lexicon file (to argv[1].lex)

#include <iostream>
#include <fstream>
#include "fries.h"

using namespace std;

fex* extractor;
map<string,int> lexicon;

void ProcessResults(const list<sentence> &ls, const list<string> &bio) {
  
  list<sentence>::const_iterator s;
  word::const_iterator a;
  vector<set<int> > features;
  vector<set<int> >::iterator w;
  set<int>::iterator f;
  int i;
  map<string,int>::iterator p;
  list<string>::const_iterator b;

  // for each sentence in list (always only one in this example)
  for (s=ls.begin(); s!=ls.end(); s++) {

    // extract sentence features. Since we gave the lexicon to the FEX at
    // creation time, we now can ask for integer-encoded features.
    features.clear();
    extractor->encode_int(*s,features);

    for (b=bio.begin(),w=features.begin();  w!=features.end();  b++,w++) {
      cout<<(*b);
      // print a line for each word, with right BIO class plus encoded features
      for (f=w->begin(); f!=w->end(); f++) {
	cout<<" "<<(*f);
      }
      cout<<endl;
    }
  }
}


int main(int argc, char* argv[]) {

  string text,form,lemma,tag,b;
  sentence av;
  list<string> bio;
  list<sentence> ls;
  char snam[512];
  char lnam[512];

  strcpy(snam, argv[1]);
  strcpy(lnam, argv[1]);

  // create feature extractor with given set of feature rules, 
  // and previously acquired lexicon (see sample program lexicon.cc)
  extractor = new fex(strcat(snam,".rgf"),strcat(lnam,".lex"));

  // read sentences. This sample program assumes input is given 
  // in the following format: Four blank separated fields per
  // line (word,lemma,PoS,BIO-tag), and one blank line separating
  // consecutive sentences.
  while (std::getline(std::cin,text)) {
    
    if (text != "") { // got a word line
      istringstream sin; sin.str(text);
      // get word form
      sin>>form; 	
      
      // build new word
      word w(form);

      // get and store right class for this word
      sin>>b; 
      bio.push_back(b);

      // append new word to sentence
      av.push_back(w);
    }
    else { // blank line, sentence end.
      ls.push_back(av);
      
      ProcessResults(ls,bio);

      bio.clear();  // clear list of BIO tags for next use
      av.clear(); // clear list of words for next use
      ls.clear(); // clear list of sentences for next use
    }
  }
  
  // last sentence (may not have blank line after it)
  ls.push_back(av);
  ProcessResults(ls,bio);
  
  int i;
  map<string,int>::iterator p;
  strcpy(snam, argv[1]);
  ofstream lex(strcat(snam,".lex"));
  for (i=0,p=lexicon.begin(); p!=lexicon.end(); i++,p++) 
    lex<<i<<" "<<p->first<<" "<<p->second<<endl;
  lex.close();
}


